package  com.ablecloud.demo;

import com.ablecloud.common.ACFilter;
import com.ablecloud.common.ACObject;
import com.ablecloud.common.ACContext;
import com.ablecloud.service.ACCronJob;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.text.SimpleDateFormat;

public class DemoCronJob extends ACCronJob {
    private static final Logger logger = LoggerFactory.getLogger(DemoCronJob.class);
    private static final String DEMO_DATA_CLASS = "light_action_data";
    private static final String DEMO_ANALYSIS_CLASS = "light_action_analysis";

    @Override
    public int run() throws Exception {
        try {
            logger.info("cron job started: " + new Date().toString());
            ACContext context = getAc().newContext("");
            long current = System.currentTimeMillis();
            long start = current - (long) 24 * 60 * 60 * 1000;
            ACFilter filterStartTime = ac.filter().whereGreaterThanOrEqualTo("time", start);
            ACFilter filterEndTime = ac.filter().andLessThanOrEqualTo("time", current);
            List<ACObject> results = ac.store(DEMO_DATA_CLASS, context)
                    .scan()
                    .select("deviceId", "action")
                    .where(filterStartTime)
                    .and(filterEndTime)
                    .groupBy("action")
                    .count()
                    .execute();

            Calendar cal = Calendar.getInstance();
            cal.add(Calendar.DAY_OF_MONTH, -1);
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
            String date = df.format(cal.getTime());
            for (ACObject zo : results) {
                Number deviceId = zo.get("deviceId");
                if (deviceId == null) {
                    logger.warn("invalid scan return: deviceId[" + deviceId + "]");
                    continue;
                }
                Number action = zo.get("action");
                Number count = zo.get("_count");
                if (action == null || count == null) {
                    logger.warn("invalid full scan return: action[" + action + "], count[" + count + "]");
                    continue;
                }
                ac.store(DEMO_ANALYSIS_CLASS, context)
                        .create("deviceId", deviceId.longValue(), "date", date, "action", action.longValue())
                        .put("total_count", count.longValue())
                        .execute();
                logger.info("device analysis finished: deviceId[" + deviceId.longValue() + "]");
            }
            logger.info("Sleeping for 1 minutes.");
            java.util.concurrent.TimeUnit.MINUTES.sleep(1);
            logger.info("cron job finished: " + new Date().toString());
            return 0;
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("Exception in cron-job finished: " + e.getMessage());
            logger.info("Sleeping for 1 minutes.");
            java.util.concurrent.TimeUnit.MINUTES.sleep(1);
            return -1;
        }
    }
}